
/*
*  Copyright (c) 2004 – 2014 Texas Instruments Incorporated
*  All rights reserved not granted herein.
*
*  Limited License. 
*
*  Texas Instruments Incorporated grants a world-wide, royalty-free,
*  non-exclusive license under copyrights and patents it now or hereafter 
*  owns or controls to make, have made, use, import, offer to sell and 
*  sell ("Utilize") this software subject to the terms herein. With respect
*  to the foregoing patent license, such license is granted solely to the 
*  extent that any such patent is necessary to Utilize the software alone. 
*  The patent license shall not apply to any combinations which include 
*  this software, other than combinations with devices manufactured 
*  by or for TI (“TI Devices”). No hardware patent is licensed hereunder.
*
*  Redistributions must preserve existing copyright notices and reproduce 
*  this license (including the above copyright notice and the disclaimer 
*  and (if applicable) source code license limitations below) in the 
*  documentation and/or other materials provided with the distribution
*
*  Redistribution and use in binary form, without modification, are permitted 
*  provided that the following conditions are met:
*
* 	No reverse engineering, decompilation, or disassembly of this software
*	is permitted with respect to any software provided in binary form.
*
* 	any redistribution and use are licensed by TI for use only with TI Devices.
*
* 	Nothing shall obligate TI to provide you with source code for the software 
*	licensed and provided to you in object code.
*
*  If software source code is provided to you, modification and redistribution 
*  of the source code are permitted provided that the following conditions are met:
*
*	any redistribution and use of the source code, including any resulting 
*	derivative works, are licensed by TI for use only with TI Devices.
*
*	any redistribution and use of any object code compiled from the source code 
*	and any resulting derivative works, are licensed by TI for use only 
*	with TI Devices.
*
*  Neither the name of Texas Instruments Incorporated nor the names of its 
*  suppliers may be used to endorse or promote products derived from 
*  this software without specific prior written permission.
*
*  DISCLAIMER.
*
*	THIS SOFTWARE IS PROVIDED BY TI AND TI’S LICENSORS "AS IS" 
*	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
*	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
*	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TI’S LICENSORS 
*	BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
*	OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
*	OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
*	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
*	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
*	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
*	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//
// Common ARC algorithm interface declarations
//

/*
 *  This header defines all types, constants, and functions shared by all
 *  implementations of the ARC (Asynchronous sample-Rate Conversion) algorithm.
 */

#ifndef IARC_
#define IARC_

#include <ialg.h>
#include  <std.h>
#include <xdas.h>

#include "icom.h"
#include "paftyp.h"

/*
 *  ======== IARC_Obj ========
 *  Every implementation of IARC *must* declare this structure as
 *  the first member of the implementation's object.
 */
typedef struct IARC_Obj {
    struct IARC_Fxns *fxns;    /* function list: standard, public, private */
} IARC_Obj;

/*
 *  ======== IARC_Handle ========
 *  This type is a pointer to an implementation's instance object.
 */
typedef struct IARC_Obj *IARC_Handle;

/*
 *  ======== IARC_Status ========
 *  This Status structure defines the parameters that can be changed or read
 *  during real-time operation of the algorithm.  This structure is actually
 *  instantiated and initialized in iarc.c.
 */
typedef volatile struct IARC_Status {
    Int size;             /* This value must always be here, and must be set to the
                             total size of this structure in 8-bit bytes, as the
                             sizeof() operator would do. */
    XDAS_Int8 mode;       /* This is the 8-bit ARC Mode Control Register.  All
                             Algorithms must have a mode control register.  */
    XDAS_Int8 unused1[3];
    XDAS_UInt32 inputsPerOutputQ24; /* (mixed) no. of inputs per output sample */
    XDAS_UInt32 outputsRemainingQ24; /* (mixed) no. of (internally buffered) outputs */
    XDAS_UInt32 maxInputsPerOutputQ24; /* Max allowed no. of inputs per output sample */
} IARC_Status;

/*
 *  ======== IARC_Config ========
 *  Config structure defines the parameters that cannot be changed or read
 *  during real-time operation of the algorithm.
 */
typedef struct IARC_Config {
    XDAS_Int8 nChannels;
    XDAS_Int8 nTaps;
    XDAS_Int16 mask;
    /// FilterCoefs **coefs;
    PAF_AudioData *pState;
} IARC_Config;

/*
 *  ======== IARC_Params ========
 *  This structure defines the parameters necessary to create an
 *  instance of a ARC object.
 *
 *  Every implementation of IARC *must* declare this structure as
 *  the first member of the implementation's parameter structure.
 *  This structure is actually instantiated and initialized in iarc.c.
 */
typedef struct IARC_Params {
    Int size;
    const IARC_Status *pStatus;
    IARC_Config config;
} IARC_Params;

/*
 *  ======== IARC_PARAMS ========
 *  Default instance creation parameters (defined in iarc.c)
 */
extern const IARC_Params IARC_PARAMS;

/*
 *  ======== IARC_Fxns ========
 *  All implementation's of ARC must declare and statically
 *  initialize a constant variable of this type.
 *
 *  By convention the name of the variable is ARC_<vendor>_IARC, where
 *  <vendor> is the vendor name.
 */
typedef struct IARC_Fxns {
    /* public */
    IALG_Fxns   ialg;
    Int         (*reset)(IARC_Handle, PAF_AudioFrame *);
    Int         (*apply)(IARC_Handle, PAF_AudioFrame *);
    /* private */
} IARC_Fxns;

/*
 *  ======== IARC_Cmd ========
 *  The Cmd enumeration defines the control commands for the ARC control method.
 */
typedef enum IARC_Cmd {
    IARC_NULL                    = ICOM_NULL,
    IARC_GETSTATUSADDRESS1       = ICOM_GETSTATUSADDRESS1,
    IARC_GETSTATUSADDRESS2       = ICOM_GETSTATUSADDRESS2,
    IARC_GETSTATUS               = ICOM_GETSTATUS,
    IARC_SETSTATUS               = ICOM_SETSTATUS
} IARC_Cmd;

#endif  /* IARC_ */
