/*
*  Copyright (c) 2004  2014 Texas Instruments Incorporated
*  All rights reserved not granted herein.
*
*  Limited License. 
*
*  Texas Instruments Incorporated grants a world-wide, royalty-free,
*  non-exclusive license under copyrights and patents it now or hereafter 
*  owns or controls to make, have made, use, import, offer to sell and 
*  sell ("Utilize") this software subject to the terms herein. With respect
*  to the foregoing patent license, such license is granted solely to the 
*  extent that any such patent is necessary to Utilize the software alone. 
*  The patent license shall not apply to any combinations which include 
*  this software, other than combinations with devices manufactured 
*  by or for TI (TI Devices). No hardware patent is licensed hereunder.
*
*  Redistributions must preserve existing copyright notices and reproduce 
*  this license (including the above copyright notice and the disclaimer 
*  and (if applicable) source code license limitations below) in the 
*  documentation and/or other materials provided with the distribution
*
*  Redistribution and use in binary form, without modification, are permitted 
*  provided that the following conditions are met:
*
* 	No reverse engineering, decompilation, or disassembly of this software
*	is permitted with respect to any software provided in binary form.
*
* 	any redistribution and use are licensed by TI for use only with TI Devices.
*
* 	Nothing shall obligate TI to provide you with source code for the software 
*	licensed and provided to you in object code.
*
*  If software source code is provided to you, modification and redistribution 
*  of the source code are permitted provided that the following conditions are met:
*
*	any redistribution and use of the source code, including any resulting 
*	derivative works, are licensed by TI for use only with TI Devices.
*
*	any redistribution and use of any object code compiled from the source code 
*	and any resulting derivative works, are licensed by TI for use only 
*	with TI Devices.
*
*  Neither the name of Texas Instruments Incorporated nor the names of its 
*  suppliers may be used to endorse or promote products derived from 
*  this software without specific prior written permission.
*
*  DISCLAIMER.
*
*	THIS SOFTWARE IS PROVIDED BY TI AND TIS LICENSORS "AS IS" 
*	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
*	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
*	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TIS LICENSORS 
*	BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
*	OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
*	OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
*	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
*	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
*	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
*	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#include <std.h>

#include <dsplink.h>
#include <notify.h>

#include <acp.h>
#include <acp_mds.h>
#include "nic_link.h"

// .............................................................................
// local and global variables

// handle for communicating with other PAF components via sequence
static ACP_Handle acp = NULL;

// status structure with which ARM communicates via alpha code
static NIC_LINK_Status nicLinkStatus =
{
  	sizeof (NIC_LINK_Status),    		// size
   	1,									// mode 
   	1,									// assertmode (preferred over mode for control))
   	0,									// assertstate
   	0, 									// deassertallow
};

// grab from afp_link.c
extern Bool PA_DSPLINK_isInitialized;

// .............................................................................
// local definitions of the number of events to generate, their sources, and
// status handlers

static int _getStat (ACP_Unit *from);
static int _handler1 (void);

typedef struct _NIC_LINK_Event
{ 
    int                event;  // event number (same number that needs to be registered by ARM app)
    int     (*handler)(void);  // handler function for this event
} _NIC_LINK_Event;

#define _NUM_EVENTS 1

static _NIC_LINK_Event nicLinkEvents[_NUM_EVENTS] =
{
    1, 
    _handler1,
};    

// -----------------------------------------------------------------------------

void nicLinkTask (void)
{
    Int        retVal;
    Int             i;


    // .........................................................................
    // one time initialization of ACP for NIC

    if (!acp) {
        ACP_MDS_init ();
        
        acp = ACP_create (&ACP_MDS_IACP, &ACP_PARAMS);
        if (!acp)
            return;

        acp->fxns->attach (acp,
                           ACP_SERIES_STD,
                           STD_BETA_ASSERT,
                           (IALG_Status *) &nicLinkStatus);
    }

    // .........................................................................
    // Do nothing if
    //       afp_link task hasn't initialized DSP link
    //       if NIC mode is disabled
    //       if NIC assertion mode is disabled
    //       if NIC has notified ARM but ARM hasn't enabled more interrupts

    if (PA_DSPLINK_isInitialized == FALSE) 
        return;
    
    if (!nicLinkStatus.mode || !nicLinkStatus.assertmode) 
        return;

	// once notification is sent then wait for host to indicate it is ready to 
    // receive more notifications
	if (nicLinkStatus.assertstate) {
		if (nicLinkStatus.deassertallow)
			nicLinkStatus.assertstate = 0;	        	 	 
		else
			return;
	}        

    // .........................................................................
	// Loop over events and query handler if notification should be sent

	for (i=0; i < _NUM_EVENTS; i++) {
        retVal = (*nicLinkEvents[i].handler) ();
        if (retVal) {
            Int status = NOTIFY_notify (ID_GPP,
                                        NIC_LINK_IPS_ID,
                                        NIC_LINK_IPS_EVENTNO_BASE,
                                        nicLinkEvents[i].event);

            // indicate we have generated notification and reset ARM flag
            //if error then likely arm side was not registered yet so ignore
            if (status == SYS_OK) {
                nicLinkStatus.assertstate   = 1;
                nicLinkStatus.deassertallow = 0;
            }
        }

        // break out of for loop since we already have one signal source
        break;
    }

    return;
} //nicLinkTask

// -----------------------------------------------------------------------------
// Event 1 handler -- triggers off of readDECSourceProgram

static int _handler1 (void)
{
    // readDECSourceProgram
    static ACP_Unit from[3] = {0xC902, 0xC224, 0x0A00};
    static int     prevStat = 0;
    int              retVal;


    retVal = _getStat (from);

    if (retVal != prevStat) {
        prevStat = retVal;
        return 1;
    }

    return 0;
} //__handler1

// -----------------------------------------------------------------------------
// General purpose alpha code reader. For user by all handlers

static int _getStat (ACP_Unit *from)
{
	ACP_Unit to[5];	    
	unsigned char type = ((from[1]>>8) & 0x7);	    
	Int readVal = -1;   

	    
	if (!(acp->fxns->sequence(acp, from, to))) {	    
		switch(type) {
			case 0:
	     	 	break;
	     	 
	     	case 1:
	     	 	break;

	     	case 2:
	     	 	readVal = (to[2] & 0xFF);
	     	 	break;

	     	case 3:
				readVal = to[3];
	     	 	break;

	     	case 4:
	     	 	readVal = to[3] | (to[4]<<16);
	     	 	break;

	     	case 5:
	     	 	break;

	     	case 6:
	     	 	break;

	     	case 7:
	     	 	break;
	     	
	     	default:
	     	 	break; 
		} 
	}
	return readVal;
} //_getStat

// -----------------------------------------------------------------------------
