
/*
*  Copyright (c) 2004 – 2014 Texas Instruments Incorporated
*  All rights reserved not granted herein.
*
*  Limited License. 
*
*  Texas Instruments Incorporated grants a world-wide, royalty-free,
*  non-exclusive license under copyrights and patents it now or hereafter 
*  owns or controls to make, have made, use, import, offer to sell and 
*  sell ("Utilize") this software subject to the terms herein. With respect
*  to the foregoing patent license, such license is granted solely to the 
*  extent that any such patent is necessary to Utilize the software alone. 
*  The patent license shall not apply to any combinations which include 
*  this software, other than combinations with devices manufactured 
*  by or for TI (“TI Devices”). No hardware patent is licensed hereunder.
*
*  Redistributions must preserve existing copyright notices and reproduce 
*  this license (including the above copyright notice and the disclaimer 
*  and (if applicable) source code license limitations below) in the 
*  documentation and/or other materials provided with the distribution
*
*  Redistribution and use in binary form, without modification, are permitted 
*  provided that the following conditions are met:
*
* 	No reverse engineering, decompilation, or disassembly of this software
*	is permitted with respect to any software provided in binary form.
*
* 	any redistribution and use are licensed by TI for use only with TI Devices.
*
* 	Nothing shall obligate TI to provide you with source code for the software 
*	licensed and provided to you in object code.
*
*  If software source code is provided to you, modification and redistribution 
*  of the source code are permitted provided that the following conditions are met:
*
*	any redistribution and use of the source code, including any resulting 
*	derivative works, are licensed by TI for use only with TI Devices.
*
*	any redistribution and use of any object code compiled from the source code 
*	and any resulting derivative works, are licensed by TI for use only 
*	with TI Devices.
*
*  Neither the name of Texas Instruments Incorporated nor the names of its 
*  suppliers may be used to endorse or promote products derived from 
*  this software without specific prior written permission.
*
*  DISCLAIMER.
*
*	THIS SOFTWARE IS PROVIDED BY TI AND TI’S LICENSORS "AS IS" 
*	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
*	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
*	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TI’S LICENSORS 
*	BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
*	OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
*	OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
*	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
*	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
*	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
*	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//
//
// GEQ interface declarations
//
//
//


/*
 *  This header defines all types, constants, and functions shared by all
 *  implementations of the ASP Example Demonstration algorithm.
 */
#ifndef IGEQ_
#define IGEQ_

#include <ialg.h>
#include  <std.h>
#include <xdas.h>

#include "icom.h"
#include "paftyp.h"

#ifndef PI
#define PI  3.14159265358979323
#endif

#define GEQ_MAX_NUM_BANDS 16 /* max number of bands */

#define GEQ_MAX_NUM_CHAN  16 /* max number of Channels */

/* #if GEQ_MAX_NUM_CHAN != PAF_MAXNUMCHAN
#   error "GEQ is available only for 16-channel builds"
#endif */

#define kInterpLen  256

/*
 * EQ state type, float or double.
 */
typedef float   IGEQ_StateSP;
typedef double  IGEQ_StateDP;

/*
 * Default type: float
 */
typedef float IGEQ_Real;

/*
 * Second order EQ types.
 */
typedef enum {
    IGEQ_Parametric,
    IGEQ_LowShelf,
    IGEQ_HighShelf,
    IGEQ_Invalid
} IGEQ_FilType;

/*
 * Generic second order EQ params.
 * Mapping functions from parameters to coefficients.
 * fs:  sampling rate
 * f0:  center or cutoff frequency
 * H:   height in dB
 * W:   width in octaves
 * g:   filter gain
 */

typedef struct {
    IGEQ_FilType type;   /* EQ type */
    IGEQ_Real fs;       /* sampling rate */
    IGEQ_Real f0;       /* cutoff or corner freq */
    IGEQ_Real W;        /* width in octaves */
    IGEQ_Real H;        /* height in dB */
    IGEQ_Real g;        /* linear gain */
} IGEQ_FilParam;


/*
 *  ======== IGEQ_Obj ========
 *  Every implementation of IGEQ *must* declare this structure as
 *  the first member of the implementation's object.
 */
typedef struct IGEQ_Obj {
    struct IGEQ_Fxns *fxns;    /* function list: standard, public, private */
} IGEQ_Obj;

/*
 *  ======== IGEQ_Handle ========
 *  This type is a pointer to an implementation's instance object.
 */
typedef struct IGEQ_Obj *IGEQ_Handle;

/*
 *  ======== IGEQ_Status ========
 *  This Status structure defines the parameters that can be changed or read
 *  during real-time operation of the algorithm.  This structure is actually
 *  instantiated and initialized in igeq.c.
 */

/*
 * Parametric band.
 */
typedef struct {
    XDAS_Int16 freq;    /* center frequency, Hz */
    XDAS_Int16 level;   /* gain in dB * 10 */
    XDAS_Int16 width;   /* width in octaves * 10 */
} IGEQ_Band;
typedef struct IGEQ_StatusBase {
    Int size;             /* This value must always be here, and must be set to the 
                             total size of this structure in 8-bit bytes, as the 
                             sizeof() operator would do. */
    XDAS_Int8 mode;       /* This is the 8-bit Mode Control Register.  All 
                             Algorithms must have a mode control register.  */

    XDAS_Int8 use;        /* This is the 8-bit Use Control Register. */
    XDAS_Int8 paramChg;   /* new parameters available if != 0 */
    XDAS_Int8 numBands;   /* No. of bands actually used */
    XDAS_Int32 enableBand;  /* Active Bands */
    XDAS_Int32 enableChan;  /* Active Channels */
} IGEQ_StatusBase;

typedef volatile struct IGEQ_Status {
    Int size;             /* This value must always be here, and must be set to the 
                             total size of this structure in 8-bit bytes, as the 
                             sizeof() operator would do. */
    XDAS_Int8 mode;       /* This is the 8-bit Mode Control Register.  All 
                             Algorithms must have a mode control register.  */

    XDAS_Int8 use;        /* This is the 8-bit Use Control Register. */
    XDAS_Int8 paramChg;   /* new parameters available if != 0 */
    XDAS_Int8 numBands;   /* No. of bands actually used */
    XDAS_Int32 enableBand;  /* Active Bands */
    XDAS_Int32 enableChan;  /* Active Channels */
    IGEQ_Band band[1];
} IGEQ_Status;



/*
 *  ======== IGEQ_Config ========
 *  Config structure defines the parameters that cannot be changed or read
 *  during real-time operation of the algorithm.
 */
typedef struct IGEQ_Config {
    Int        commonMem;
    XDAS_Int8  maxChans;     /* maximum number of channels */
    XDAS_Int8  maxBands;     /* maximum number of bands */
    XDAS_Int8  uniCoef;      /* are params same across channels */
    XDAS_Int32 applyMask;
    XDAS_Int32 mxMask;
} IGEQ_Config;

/*
 *  ======== IGEQ_Params ========
 *  This structure defines the parameters necessary to create an
 *  instance of a GEQ object.
 *
 *  Every implementation of IGEQ *must* declare this structure as
 *  the first member of the implementation's parameter structure.  This structure is actually
 *  instantiated and initialized in igeq.c.
 */
typedef struct IGEQ_Params {
    Int size;
    const IGEQ_Status *pStatus;
    IGEQ_Config config;
} IGEQ_Params;

/*
 *  ======== IGEQ_PARAMS ========
 *  Default instance creation parameters (defined in igeq.c)
 */
extern const IGEQ_Params IGEQ_PARAMS;
extern const IGEQ_Params IGEQ_PARAMS_LOU;
/* 10-Band Equalizer; Max 8 channels -- DEFAULT -- */
extern const IGEQ_Params IGEQ_PARAMS_EQX;
/* 10-Band Equalizer; Max 8 channels; 4 bands mx filters */
extern const IGEQ_Params IGEQ_PARAMS_EQX_HQ;
/* 10-Band Equalizer; Max 16 channels*/
extern const IGEQ_Params IGEQ_PARAMS_EQX_16CHN_UNICOEF;
/* 10-Band Equalizer; Max 16 channels; 4 bands mx filters */
extern const IGEQ_Params IGEQ_PARAMS_EQX_HQ_16CHN_UNICOEF;
/* 10-Band Equalizer; Max 16 channels; channel-wise control */
extern const IGEQ_Params IGEQ_PARAMS_EQX_16CHN;
/* 3-Band Equalizer; Max 8 channels */
extern const IGEQ_Params IGEQ_PARAMS_EQT;
/* 5-Band Equalizer; Max 8 channels */
extern const IGEQ_Params IGEQ_PARAMS_EQV;

/*
 *  ======== IGEQ_Fxns ========
 *  All implementation's of GEQ must declare and statically 
 *  initialize a constant variable of this type.
 *
 *  By convention the name of the variable is GEQ_<vendor>_IGEQ, where
 *  <vendor> is the vendor name.
 */
typedef struct IGEQ_Fxns {
    /* public */
    IALG_Fxns   ialg;
    Int  (*reset)(IGEQ_Handle, PAF_AudioFrame *);
    Int  (*apply)(IGEQ_Handle, PAF_AudioFrame *);

    /* private */    
    Int (*clearState)(IGEQ_Handle handle);
    Int (*applyLou)(IGEQ_Handle handle, PAF_AudioFrame * pAudioFrame);
    Int (*sweepProcSP)(IGEQ_Handle handle, IGEQ_Real *pIn,  
            IGEQ_Real *pCoef, IGEQ_Real *pCoefInc, 
            IGEQ_StateSP *pVar, Int nBands, Int nSamp);
    Int (*sweepProcDP)(IGEQ_Handle handle, IGEQ_Real *pIn,  
            IGEQ_Real *pCoef, IGEQ_Real *pCoefInc, 
            IGEQ_StateDP *pVar, Int nBands, Int nSamp);
    Int (*procSP)(IGEQ_Handle handle, IGEQ_Real *pIn,  
            IGEQ_Real *pCoef, IGEQ_StateSP *pVar, 
            Int nBands, Int nSamp);
    Int (*procDP)(IGEQ_Handle handle, IGEQ_Real *pIn,  
            IGEQ_Real *pCoef, IGEQ_StateDP *pVar, 
            Int nBands, Int nSamp);
    Int (*calcCoef)(IGEQ_Handle handle, 
        IGEQ_FilParam *pP,
        IGEQ_Real *pC);


} IGEQ_Fxns;

/*
 *  ======== IGEQ_Cmd ========
 *  The Cmd enumeration defines the control commands for the GEQ
 *  control method.
 */
typedef enum IGEQ_Cmd {
    IGEQ_NULL                    = ICOM_NULL,
    IGEQ_GETSTATUSADDRESS1       = ICOM_GETSTATUSADDRESS1,
    IGEQ_GETSTATUSADDRESS2       = ICOM_GETSTATUSADDRESS2,
    IGEQ_GETSTATUS               = ICOM_GETSTATUS,
    IGEQ_SETSTATUS               = ICOM_SETSTATUS
} IGEQ_Cmd;

#endif  /* IGEQ_ */
